# 1. 配置中心go SDK介绍
配置中心go SDK旨在降低配置中心的整合难度。开发语言是Go.

# 2. 名词解释

## 2.1 Module(模块)
代表一个服务，每个模块或者服务都有一个ID，ID是唯一的。

## 2.2 ServiceInstance
ServiceInstance代表一个服务实例，支持的服务实例列表Postgres,Mongodb,Redis和Influxdb。 Rabbitmq和Minio在开发中。

## 2.3 Router
Router代表了服务的路由，路由包含了一个名称和端口，并对外提供一个服务。每个模块会包含多个router。

Router中Address的格式为(schema)://127.0.0.1:8080/xx/xxx <br>
eg:<br>
	http://127.0.0.1:8080<br>
	127.0.0.1<br>
	127.0.0.1:8080<br>
	http://127.0.0.1:8080/path/uri<br>

router现在有三种类型
1. 普通类型的router，只需要定义name和address即可
2. 给Dashboard统一数据源用的router, 除了需要name和address外，还需要把isDashboard设置为true
3. 给Desk用的router，除了需要name和address外，需要把isPortal设置为true, 按需设置下边的参数
	- IconUrl: Desk ICON的地址
	- TitleText
	- LinkType
	- Categories

# 3. 注册流程

## 3.1 定义本服务的配置

配置中需要包含服务的ID，需要的服务实例列表以及所有的路由。
```
moduleCfg := module.ModuleConfig{
	Name: "simple-module",
	ServiceInstances: []*configcenter.ServiceInstance{
		{
			ServiceInstanceType: configcenter.ServiceInstance_Postgres,
			ServiceInstanceConfig: map[string]string{
				"role": "xxxxx",
			},
		},
		{
			ServiceInstanceType: configcenter.ServiceInstance_Redis,
		},
		{
			ServiceInstanceType: configcenter.ServiceInstance_Influxdb,
		},
		{
			ServiceInstanceType: configcenter.ServiceInstance_RabbitMQ,
		},
	},
	Routers: []*configcenter.Router{
		{
			Name:    "test",
			Address: "http://127.0.0.1:8080",
		},
	},
}
```

然后初始化一个服务
```
m := module.NewModule(moduleCfg, ctx)
```

## 3.2 连接配置中心

需要在协程中运行
```
m.LinkConfigCenter()
```

# 4. SDK 函数说明

当连接到配置中心后，可以使用下面的函数获取配置信息。

## 4.1 获取所有的公共配置

返回所有的公共配置，注意这个函数不是协程安全的，不能写入
```
module.GetPublicConfigsUnsafe() map[string]string
```

## 4.2 获取单个公共配置

根据key返回单个公共配置，第二个参数代表公共配置是否存在，true代表存在
```
module.GetPublicConfig(key string) (string, bool)
```

## 4.3 获取服务实例的列表

返回申请的服务实例的列表，注意这个函数不是协程安全的，不能写入
```
module.GetServiceInstancesUnsafe() map[string]string 
```

## 4.4 获取单个服务实例列表

返回单个服务实例，第二个参数代表服务实例是否存在，true代表存在
```
module.GetServiceInstance(key string) (string, bool)
```

## 4.5 获取服务列表

返回服务列表，注意这个函数不是协程安全的，不能写入
```
module.GetAllServicesUnsafe() map[string]*configcenter.ServiceInfo
```

## 4.6 获取某个服务的Router列表

返回某个服务所有的router，注意这个函数不是协程安全的，不能写入
```
module.GetRouters(serviceName string) (map[string]*configcenter.Router, bool)
```

## 4.7 获取某个服务的某个路由

获取某个服务的某个路由，如果不存在，第二个参数返回false
```
GetRouter(serviceName string, routerName string) (*configcenter.Router, bool)
```

## 4.8 获取当前模块的状态
获取当前的状态，状态有以下几种<br>
Connecting 表示正在连接配置中心 <br>
DisConnected 表示和配置中心的连接断开 <br>
Reconnecting 表示和配置中心正在断线重连 <br>
Connected 表示和配置中心已经连接<br>
Destry   表示模型已经退出 <br>

```
GetStatus() ModuleStatus
```

## 4.9 更新模块的配置信息
```
UpdateConfig(cfg ModuleConfig)
```

## 4.10 获取某个服务的public address
获取某个服务的某个路由的public address，如果不存在，第二个参数返回false
```
GetRouterPublicAddress(serviceName string, routerName string) (string, bool)
```

# 5 如何使用

## 5.1 在程序中引入库

```
go get -u gitlab.wise-paas.com/sdk/configcenter@{branch_name}
go mod tidy
```

branch_name可以为空，空代表master分支, branch_name可以是分支的名称，也可以是tag的名称