package connectstring

import (
	"testing"

	"github.com/stretchr/testify/assert"
)

func TestParseConnectString(t *testing.T) {
	type testCase struct {
		connectStr string
		cfgs       map[string]string
	}

	testCases := []testCase{
		{
			connectStr: "type=dccs;deviceid=xxxx;dccs_url=xxxxx;dccs_key=xxxxx",
			cfgs: map[string]string{
				"type":     "dccs",
				"deviceid": "xxxx",
				"dccs_url": "xxxxx",
				"dccs_key": "xxxxx",
			},
		},
		{
			connectStr: "type=websocket;deviceid=xxxx;url=xxxxx;secret=xxxxx",
			cfgs: map[string]string{
				"type":     "websocket",
				"deviceid": "xxxx",
				"url":      "xxxxx",
				"secret":   "xxxxx",
			},
		},
	}

	for _, testCase := range testCases {
		cfgs, err := ParseConnectString(testCase.connectStr)
		if err != nil {
			t.Error(err)
			return
		}

		assert.Equal(t, cfgs, testCase.cfgs)
	}
}
