package dccs

import (
	"encoding/json"
	"fmt"
	"io/ioutil"
	"net/http"
)

func getDCCSResponse(dccsAddress, dccsKey string) ([]byte, error) {
	url := fmt.Sprintf("%s/v1/serviceCredentials/%s", dccsAddress, dccsKey)
	res, err := http.Get(url)
	if err != nil {
		return nil, err
	}
	defer res.Body.Close()

	body, err := ioutil.ReadAll(res.Body)
	if err != nil {
		return nil, err
	}

	if res.StatusCode != http.StatusOK {
		return nil, fmt.Errorf(string(body))
	}

	return body, nil
}

func parseDCCSResponse(body []byte) (string, string, string, error) {
	var response struct {
		ServiceName string
		ServiceHost string
		Credential  struct {
			Password      string
			Username      string
			ExternalHosts string `json:"externalHosts"`
			Hostname      string `json:"hostname"`
			Protocols     map[string]struct {
				Ssl      bool
				Username string
				Password string
				Port     int
			}
		}
	}

	if err := json.Unmarshal([]byte(body), &response); err != nil {
		return "", "", "", err
	}

	return fmt.Sprintf("tcp://%s:%d", response.ServiceHost, response.Credential.Protocols["mqtt"].Port),
		response.Credential.Protocols["mqtt"].Username,
		response.Credential.Protocols["mqtt"].Password, nil
}

func GetMQTTCredential(dccsUrl string, dccsKey string) (string, string, string, error) {
	resp, err := getDCCSResponse(dccsUrl, dccsKey)
	if err != nil {
		return "", "", "", err
	}

	return parseDCCSResponse(resp)
}
