package main

import (
	"context"
	"fmt"
	"time"

	"github.com/wh8199/log"

	mqtt "github.com/eclipse/paho.mqtt.golang"
	"gitlab.wise-paas.com/sdk/configcenter/dsn"
	"gitlab.wise-paas.com/sdk/configcenter/pkg/configcenter"
	"gitlab.wise-paas.com/sdk/configcenter/pkg/module"
)

func main() {
	moduleCfg := module.ModuleConfig{
		Name: "simple-module",
		ServiceInstances: []*configcenter.ServiceInstance{
			{
				ServiceInstanceType: configcenter.ServiceInstance_RabbitMQ,
			},
		},
	}

	ctx, cancelFunc := context.WithCancel(context.Background())
	defer cancelFunc()

	m := module.NewModule(moduleCfg, ctx)

	go m.LinkConfigCenter()

	var (
		mqttDSN string
		ok      bool
	)

	for {
		mqttDSN, ok = m.GetServiceInstance(configcenter.ServiceInstance_RabbitMQ.String())
		if ok {
			break
		}

		time.Sleep(time.Second)
	}

	connInfo, err := dsn.ParseDSN(mqttDSN)
	if err != nil {
		log.Fatal(err)
	}
	if err := connInfo.ToMqtt(); err != nil {
		log.Fatal(err)
	}

	broker := fmt.Sprintf("%s:%d", connInfo.Host, connInfo.Port)

	opts := mqtt.NewClientOptions().AddBroker(broker).SetClientID("gotrivial")
	opts.SetKeepAlive(2 * time.Second)
	opts.SetPingTimeout(1 * time.Second)

	c := mqtt.NewClient(opts)
	if token := c.Connect(); token.Wait() && token.Error() != nil {
		log.Fatal("connect to mqtt broker failed: ", token.Error())
	}
	defer c.Disconnect(1000)
	log.Infof("connect to mqtt broker [%s] successfully", broker)

	ch := make(chan struct{}, 10)

	c.Subscribe("test", 1, func(c mqtt.Client, m mqtt.Message) {
		log.Info(string(m.Payload()))
		ch <- struct{}{}
	})

	go func() {
		c.Publish("test", 1, false, "hello")
	}()

	<-ch
}
