package module

import (
	"sync"

	"github.com/bytedance/gopkg/util/gopool"
)

type CheckCandidate func(host string, port uint16) bool

func FindCandidate(checkCandidate CheckCandidate, min, max uint16) []uint16 {
	return findCandidateWithBinary(checkCandidate, min, max)
}

func findCandidateWithBinary(checkCandidate CheckCandidate, min, max uint16) []uint16 {
	ret := []uint16{}
	lock := &sync.Mutex{}
	ip := "127.0.0.1"
	p := gopool.NewPool("threadpool", 100, gopool.NewConfig())

	wg := &sync.WaitGroup{}
	for i := min; i <= max; i++ {
		task := &Task{ip, i, checkCandidate, &ret, lock}
		wg.Add(1)

		p.Go(func() {
			defer wg.Done()
			task.Run()
		})
	}
	wg.Wait()

	return ret
}

type Task struct {
	ip             string
	port           uint16
	checkCandidate CheckCandidate
	result         *[]uint16
	lock           *sync.Mutex
}

func (t *Task) Run() {
	if t.checkCandidate(t.ip, uint16(t.port)) {
		t.lock.Lock()
		*(t.result) = append(*(t.result), uint16(t.port))
		t.lock.Unlock()
	}
}
