package portrange

import (
	"errors"
	"strconv"
	"strings"
)

var (
	ErrInvalidPortRange = errors.New("invalid port range, support formats: [1-10]")
)

func ParsePortRange(portRange string) (uint16, uint16, error) {
	if strings.Contains(portRange, "-") {
		index := strings.Index(portRange, "-")

		minStr := portRange[:index]
		maxStr := portRange[index+1:]

		min, err := strconv.Atoi(minStr)
		if err != nil {
			return 0, 0, ErrInvalidPortRange
		}

		max, err := strconv.Atoi(maxStr)
		if err != nil {
			return 0, 0, ErrInvalidPortRange
		}

		return uint16(min), uint16(max), nil
	} else {
		return 0, 0, ErrInvalidPortRange
	}
}
