package util

import (
	"testing"

	"github.com/stretchr/testify/assert"
)

func TestCopyStruct(t *testing.T) {
	type A struct {
		Name string
		Age  int
	}

	a := A{
		Name: "zhangsan",
		Age:  10,
	}

	b := Copy(a)
	assert.Equal(t, a, b)
}

func TestCopyMapStruct(t *testing.T) {
	type A struct {
		Name string
		Age  int
	}

	mapA := map[string]A{
		"a": A{
			Name: "zhangsan",
			Age:  10,
		},
		"b": A{
			Name: "lisi",
			Age:  11,
		},
	}

	mapB := Copy(mapA)
	assert.Equal(t, mapA, mapB)
}

func TestCopyToTargetStruct(t *testing.T) {
	type A struct {
		Name string
		Age  int
	}

	a := A{
		Name: "zhangsan",
		Age:  10,
	}

	b := A{}

	CopyToTarget(a, &b)
	assert.Equal(t, a, b)
}

func TestCopyToTargetMap(t *testing.T) {
	type A struct {
		Name string
		Age  int
	}

	a := map[string]A{
		"test1": A{
			Name: "test1",
			Age:  10,
		},
		"test2": A{
			Name: "test2",
			Age:  20,
		},
	}

	b := map[string]A{}

	CopyToTarget(a, &b)

	assert.Equal(t, a, b)
}

func TestCopyToTargetSlice(t *testing.T) {
	type A struct {
		Name string
		Age  int
	}

	a := []A{
		{
			Name: "test1",
			Age:  10,
		},
		{
			Name: "test2",
			Age:  20,
		},
	}

	b := []A{}

	CopyToTarget(&a, &b)
	assert.Equal(t, a, b)
}

func BenchmarkCopyToTargetMap(bench *testing.B) {
	type A struct {
		Name string
		Age  int
	}

	a := map[string]A{
		"test1": {
			Name: "test1",
			Age:  10,
		},
		"test2": {
			Name: "test2",
			Age:  20,
		},
	}

	b := map[string]A{}

	bench.ResetTimer()

	for i := 0; i < bench.N; i++ {
		CopyToTarget(a, &b)
	}
}
